require 'jenkins_util/version'
require 'jenkins_util/command_line_script'
require 'jenkins_util/keychain_util'
require 'jenkins_util/simulator_util'
require 'jenkins_util/xcode_util'
require 'jenkins_util/argument_handler'
require 'jenkins_util/logger_util'
require 'jenkins_util/process_util'
require 'jenkins_util/dropbox_util'
require 'jenkins_util/zip_util'

module JenkinsUtil
  include LoggerUtil

  args = ArgumentHandler.new

  LoggerUtil.verbose?(args.verbose)

  # KeychainUtil
  unless args.keychain.nil? && args.keychain_password.nil?
    KeychainUtil.unlock_keychain(args.keychain, args.keychain_password, args.code_signing_identities)
  end

  # DropboxUtil
  unless args.dropbox_sources.nil? || args.dropbox_sources.empty? || args.dropbox_destination.nil?
    DropboxUtil.upload(args.dropbox_sources, args.dropbox_destination, args.dropbox_flatten, args.dropbox_root)
  end

  # XcodeUtil
  SimulatorUtil.reset_all_simulators if args.kill_simulators

  if !args.xcode_project_path.nil? && !args.xcode_target.nil? && !args.xcode_build_configuration.nil?
    unless args.xcode_bundle_identifier.nil?
      XcodeUtil.project_bundle_identifier(args.xcode_project_path,
                                          args.xcode_target,
                                          args.xcode_build_configuration,
                                          args.xcode_bundle_identifier)
    end

    unless args.xcode_build_number.nil?
      XcodeUtil.project_bundle_version(args.xcode_project_path,
                                       args.xcode_target,
                                       args.xcode_build_configuration,
                                       args.xcode_build_number)
    end
  end

  # Not using LoggerUtil so there is no time stamp
  puts XcodeUtil.team_id_from_plist(args.xcode_export_plist) unless args.xcode_export_plist.nil?
  puts XcodeUtil.build_name(args.xcode_archive_plist) unless args.xcode_archive_plist.nil?

  if !args.xcode_project_path.nil? && !args.xcode_set_manual_provisioning_style.nil?
    XcodeUtil.project_provisioning_style(args.xcode_project_path, !args.xcode_set_manual_provisioning_style)
  end

  if (!args.zip_sources.empty? || !args.zip_archive.nil?) && !args.zip_destination.nil?
    ZipUtil.compress(*args.zip_sources, args.zip_destination) unless args.zip_sources.empty?
    ZipUtil.uncompress(args.zip_archive, args.zip_destination) unless args.zip_archive.nil?
  end

  # Version
  LoggerUtil.log.info(VERSION) if args.version
end
