require 'jenkins_util/process_util'
require 'jenkins_util/command_line_script'
require 'jenkins_util/logger_util'
require 'json'

class SimulatorError
  attr_reader :code, :message, :udid

  def initialize(code, message, udid)
    @code = code
    @message = message
    @udid = udid
  end

  def to_s
    format('code: %d, udid %s, message: %s', @code, @udid, @message)
  end
end

class SimulatorUtil
  DEVICE_STATE_INVALID = 159
  DEVICE_ALREADY_SHUTDOWN = 163

  def initialize
  end

  def self.devices_by_os
    process = CommandLineScript.new('xcrun simctl list --json devices')

    json_string = ''
    process.stdout.each do |fragment|
      json_string += fragment
    end
    json = JSON.parse(json_string)

    json['devices']
  end

  def self.devices
    flattened_devices = []

    devices_by_os.each do |os, devices|
      devices.each do |device|
        device['os'] = os
        flattened_devices << device
      end
    end

    flattened_devices
  end

  def self.devices_by_udid
    devices_by_udid = {}

    devices.each do |device|
      devices_by_udid[device['udid']] = device
    end

    devices_by_udid
  end

  def self.boot_device(udid)
    process = CommandLineScript.new("xcrun simctl boot #{udid}")

    LoggerUtil.fatal("Could not boot up device: #{udid}") if process.exit_status.nonzero? &&
                                                             process.exit_status != DEVICE_STATE_INVALID
  end

  def self.shutdown_device(udid)
    process = CommandLineScript.new("xcrun simctl shutdown #{udid}")

    LoggerUtil.fatal("Could not shutdown device: #{udid}") if process.exit_status.nonzero? &&
                                                              process.exit_status != DEVICE_STATE_INVALID &&
                                                              process.exit_status != DEVICE_ALREADY_SHUTDOWN
  end

  def self.reset_device(udid)
    process = CommandLineScript.new("xcrun simctl erase #{udid}")

    LoggerUtil.fatal("Could not reset device: #{udid}") if process.exit_status.nonzero? &&
                                                           process.exit_status != DEVICE_STATE_INVALID
  end

  def self.reset_all_simulators
    CommandLineScript.new('killall "Simulator"')
    CommandLineScript.new('killall "Simulator (Watch)"')

    process = CommandLineScript.new('xcrun simctl shutdown all')
    LoggerUtil.fatal('Could not shutdown all simulators') if process.exit_status.nonzero?

    process = CommandLineScript.new('xcrun simctl erase all')
    LoggerUtil.fatal('Could not erase all simulators') if process.exit_status.nonzero?
  end
end
